@@ -1,5 +1,5 @@
 use std::fmt;
-use std::io::Error as IoError;
+use std::io;
 
 use ipc_channel::Error as IpcError;
 use serde::{Deserialize, Serialize};
@@ -8,18 +8,57 @@ use serde::{Deserialize, Serialize};
 ///
 /// This contains the marshalled panic information so that it can be used
 /// for other purposes.
+///
+/// This is similar to `std::panic::PanicInfo` but can cross process boundaries.
 #[derive(Serialize, Deserialize)]
-pub struct Panic {
+pub struct PanicInfo {
     msg: String,
+    pub(crate) location: Option<Location>,
     #[cfg(feature = "backtrace")]
     pub(crate) backtrace: Option<backtrace::Backtrace>,
 }
 
-impl Panic {
+/// Location of a panic.
+///
+/// This is similar to `std::panic::Location` but can cross process boundaries.
+#[derive(Serialize, Deserialize, Debug)]
+pub struct Location {
+    file: String,
+    line: u32,
+    column: u32,
+}
+
+impl Location {
+    pub(crate) fn from_std(loc: &std::panic::Location) -> Location {
+        Location {
+            file: loc.file().into(),
+            line: loc.line(),
+            column: loc.column(),
+        }
+    }
+
+    /// Returns the name of the source file from which the panic originated.
+    pub fn file(&self) -> &str {
+        &self.file
+    }
+
+    /// Returns the line number from which the panic originated.
+    pub fn line(&self) -> u32 {
+        self.line
+    }
+
+    /// Returns the column from which the panic originated.
+    pub fn column(&self) -> u32 {
+        self.column
+    }
+}
+
+impl PanicInfo {
     /// Creates a new panic object.
-    pub(crate) fn new(s: &str) -> Panic {
-        Panic {
+    pub(crate) fn new(s: &str) -> PanicInfo {
+        PanicInfo {
             msg: s.into(),
+            location: None,
             #[cfg(feature = "backtrace")]
             backtrace: None,
         }
@@ -30,6 +69,11 @@ impl Panic {
         self.msg.as_str()
     }
 
+    /// Returns the panic location.
+    pub fn location(&self) -> Option<&Location> {
+        self.location.as_ref()
+    }
+
     /// Returns a reference to the backtrace.
     ///
     /// Typically this backtrace is already resolved because it's currently
@@ -40,10 +84,11 @@ impl Panic {
     }
 }
 
-impl fmt::Debug for Panic {
+impl fmt::Debug for PanicInfo {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        f.debug_struct("Panic")
+        f.debug_struct("PanicInfo")
             .field("message", &self.message())
+            .field("location", &self.location())
             .field("backtrace", &{
                 #[cfg(feature = "backtrace")]
                 {
@@ -58,7 +103,7 @@ impl fmt::Debug for Panic {
     }
 }
 
-impl fmt::Display for Panic {
+impl fmt::Display for PanicInfo {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(f, "{}", self.msg)
     }
@@ -75,14 +120,15 @@ pub struct SpawnError {
 #[derive(Debug)]
 enum SpawnErrorKind {
     Ipc(IpcError),
-    Io(IoError),
-    Panic(Panic),
+    Io(io::Error),
+    Panic(PanicInfo),
     Cancelled,
+    TimedOut,
 }
 
 impl SpawnError {
     /// If a panic ocurred this returns the captured panic info.
-    pub fn panic_info(&self) -> Option<&Panic> {
+    pub fn panic_info(&self) -> Option<&PanicInfo> {
         if let SpawnErrorKind::Panic(ref info) = self.kind {
             Some(info)
         } else {
@@ -90,11 +136,40 @@ impl SpawnError {
         }
     }
 
+    /// True if this error comes from a panic.
+    pub fn is_panic(&self) -> bool {
+        self.panic_info().is_some()
+    }
+
+    /// True if this error indicates a cancellation.
+    pub fn is_cancellation(&self) -> bool {
+        if let SpawnErrorKind::Cancelled = self.kind {
+            true
+        } else {
+            false
+        }
+    }
+
+    /// True if this error indicates a timeout
+    pub fn is_timeout(&self) -> bool {
+        if let SpawnErrorKind::TimedOut = self.kind {
+            true
+        } else {
+            false
+        }
+    }
+
     pub(crate) fn new_cancelled() -> SpawnError {
         SpawnError {
             kind: SpawnErrorKind::Cancelled,
         }
     }
+
+    pub(crate) fn new_timeout() -> SpawnError {
+        SpawnError {
+            kind: SpawnErrorKind::TimedOut,
+        }
+    }
 }
 
 impl std::error::Error for SpawnError {
@@ -104,6 +179,7 @@ impl std::error::Error for SpawnError {
             SpawnErrorKind::Io(ref err) => Some(&*err),
             SpawnErrorKind::Panic(_) => None,
             SpawnErrorKind::Cancelled => None,
+            SpawnErrorKind::TimedOut => None,
         }
     }
 }
@@ -115,6 +191,7 @@ impl fmt::Display for SpawnError {
             SpawnErrorKind::Io(_) => write!(f, "process spawn error: i/o error"),
             SpawnErrorKind::Panic(ref p) => write!(f, "process spawn error: panic: {}", p),
             SpawnErrorKind::Cancelled => write!(f, "process spawn error: call cancelled"),
+            SpawnErrorKind::TimedOut => write!(f, "process spawn error: timed out"),
         }
     }
 }
@@ -127,16 +204,16 @@ impl From<IpcError> for SpawnError {
     }
 }
 
-impl From<IoError> for SpawnError {
-    fn from(err: IoError) -> SpawnError {
+impl From<io::Error> for SpawnError {
+    fn from(err: io::Error) -> SpawnError {
         SpawnError {
             kind: SpawnErrorKind::Io(err),
         }
     }
 }
 
-impl From<Panic> for SpawnError {
-    fn from(panic: Panic) -> SpawnError {
+impl From<PanicInfo> for SpawnError {
+    fn from(panic: PanicInfo) -> SpawnError {
         SpawnError {
             kind: SpawnErrorKind::Panic(panic),
         }
